<?php

namespace DXIM;

class VehicleEmission {

    public static $units = [
        'co2' => "g/km",
        'electricity' => 'kWh/100km',
        'fuel' => 'l/100km',
        'gas' => 'kg/100km',
        'km' => 'km'
    ];

    protected $co2_emission_class = [
        'A' => [0,0],
        'B' => [1,95],
        'C' => [96,115],
        'D' => [116,135],
        'E' => [136,155],
        'F' => [156,175],
        'G' => [175,10000000000],
    ];

    public static $no_value = "n.V.";

    protected $error = [];

    protected $rules = [
        [
            'type' => ['PETROL', 'DIESEL', 'LPG', 'CNG', 'GAS'],
            'required' => [
                'consumption' => [
                    'wltp_consumption_fuel_combined'
                ],
                'emission' => [
                    'wltp_co2_emission_combined',
                ],
                'co2_classes' => [
                    'wltp_co2_class' => ['depends' => 'wltp_co2_emission_combined']
                ],
            ],
            'recommended' => [
                'wltp_consumption_fuel_slow',
                'wltp_consumption_fuel_medium',
                'wltp_consumption_fuel_fast',
                'wltp_consumption_fuel_very_fast',
            ]
        ],
        [
            'type' => ['ELECTRICITY'],
            'required' => [
                'consumption' => [
                    'wltp_consumption_power_combined',
                ],
                'emission' => [
                    'wltp_co2_emission_combined',
                ],
                'co2_classes' => [
                    'wltp_co2_class' => ['depends' => 'wltp_co2_emission_combined']
                ],
                'wltp_electric_range'
            ],

            'recommended' => [
                'wltp_consumption_power_slow',
                'wltp_consumption_power_medium',
                'wltp_consumption_power_fast',
                'wltp_consumption_power_very_fast',
            ],
        ],
        [
            'type' => ['HYBRID', 'HYBRID_DIESEL', 'HYBRID_ALL'],
            'required' => [
                'consumption' => [
                    'wltp_consumption_fuel_combined_weighted',
                    'wltp_consumption_power_combined_weighted',
                    'wltp_consumption_fuel_combined_low_battery',
                ],
                'emission' => [
                    'wltp_co2_emission_combined_weighted',
                ],
                'co2_classes' => [
                    'wltp_co2_class_weighted_combined' => ['depends' => 'wltp_co2_emission_combined_weighted'],
                    'wltp_co2_class_low_battery' => ['depends' => 'wltp_co2_emission_low_battery']
                ],
                'wltp_electric_range_eaer'
            ],

            'recommended' => [
                'wltp_consumption_fuel_slow',
                'wltp_consumption_fuel_medium',
                'wltp_consumption_fuel_fast',
                'wltp_consumption_fuel_very_fast',
            ]
        ],
    ];

    public function get_error() {
        return $this->error;
    }

    /**
     *
     * @param $meta // vehicle post meta array
     * @return bool
     */
    public function is_allowed($meta) {#
        $emission = $this->get_vehicle_emission($meta);
        if(empty($emission)) {
            $this->error[] = "Emissionsregeln zu Fahrzeug nicht gefunden";
            return false;
        }

        // Wenn es sich nicht um einen PKW handelt => erlauben
        $class = VehicleCF::getFromMetaArray($meta['class'] ?? "");
        $to_check = ['Car'];
        $classes = [];
        foreach(VehicleClass::_getAll() as $tmp) {
            $classes[] = $tmp['key'];
        }
        if(!in_array($class, $to_check) && in_array($class, $classes)) return true;


        // wenn keine Erstzulassung angegeben, wird heute = Neuwagen angenommen
        $first_registration = VehicleCF::getFromMetaArray($meta['specific_first_registration'] ?? "",date("Y-m"));
        $time = strtotime($first_registration."-01");
        $date_allowed = strtotime(date("Y-m",strtotime("-8 months")));

        // Wenn keine Laufleistung angegeben wird 0 = Neuwagen angenommen
        $mileage = VehicleCF::getFromMetaArray($meta['specific_mileage'] ?? 0, "0");

        // Fahrzeug muss nicht geprüft werden, da kein Neuwagen
        if($time < $date_allowed && $mileage > 1000) return true;

        foreach($emission['required'] as $field) {
            if($field['value'] === "") {
                $this->error[] = "mind. Feld ".$field['label']." ".$field['filterkey']." leer";
                return false;
            }
        }

        return true;
    }

    /**
     * @param $emission
     * @return string|null
     */
    public function calculate_co2_class($emission) {
        if($emission === "") { return null; }

        foreach($this->co2_emission_class as $class => $values) {
            if($emission >= $values[0] && $emission <= $values[1]) return $class;
        }

        return null;
    }

    /**
     * @param $meta
     * @return mixed
     */
    public function fill_co2_classes_for_import($meta) {
        $fuel = VehicleCF::getFromMetaArray($meta['specific_fuel']);
        $rule = $this->get_rule($fuel);
        if(empty($rule)) return $meta;

        // Bei Elektro darf emission = 0 sein wenn nicht angegeben
        if($fuel == "ELECTRICITY" && !isset($meta['specific_wltp_co2_emission_combined'])) $meta['specific_wltp_co2_emission_combined'] = 0;


        foreach($rule['required']['co2_classes'] as $field_name => $field_definition) {
            $field = VehicleFields::getField($field_name);
            $field_meta_key = $field['meta_query']['key'];

            $depends_on_field = VehicleFields::getField($field_definition['depends']);
            $depends_meta_key = $depends_on_field['meta_query']['key'];

            if(empty($meta[$field_meta_key])) {
                $calc_value = $meta[$depends_meta_key] ?? "";
                #print $field_meta_key." is required but empty - versuche zu füllen anhand: ".$depends_meta_key." (Wert: ".$calc_value.")<br />";
                $meta[$field_meta_key] = $this->calculate_co2_class($calc_value);
                #print "Wert ist nun: ".$meta[$field_meta_key]."<br />";
            }
        }

        return $meta;
    }


    /**
     * @param $vehicle
     * @return array
     */
    public function get_vehicle_emission(&$meta) {
        $fuel = $meta['specific_fuel'] ?? null;
        if(is_array($fuel)) $fuel = $fuel[0];

        $rule = $this->get_rule($fuel);
        if(empty($rule)) return [];

        $emission['required'] = $this->get_fields_data($meta, $rule['required'], $fuel);
        $emission['recommended'] = $this->get_fields_data($meta, $rule['recommended'], $fuel);

        return $emission;
    }

    /**
     * @param $fuel
     * @return array|array[]|mixed
     */
    private function get_rule($fuel) {
        foreach($this->rules as $rule) {
            if(in_array($fuel, $rule['type'])) return $rule;
        }
        return [];
    }

    /**
     * @param $fields
     * @return array
     */
    private function _get_field_names($fields) {
        $fields_clean = [];
        foreach($fields as $field) {
            if(is_array($field)) {
                foreach($field as $key => $tmp) {
                    if(is_array($tmp)) {
                        $fields_clean[] = $key;
                    } else $fields_clean[] = $tmp;
                }
            }
            else $fields_clean[] = $field;
        }
        return $fields_clean;
    }

    /**
     * @param $fields
     * @param $fuel
     * @return void
     */
    private function get_fields_data(&$meta, $fields, $fuel) {
        $emission = [];
        #wpcs_p($vehicle);
        #wpcs_p($fields);
        $fields = $this->_get_field_names($fields);

        foreach($fields as $field_slug) {
            $field = VehicleFields::getField($field_slug);
            $meta_key = $field['meta_query']['key'];
            $meta_value = VehicleCF::getFromMetaArray($meta[$meta_key] ?? "");
            $value = $meta_value;
            if($field['field_type'] == "decimal") $value = $this->format_value($value);
            $field_unit = $field['unit'] ?? "";

            // bei Antrieb Gas die einheit von liter auf kg ändern
            if(($fuel == "CNG" || $fuel == "GAS") && $field_unit == "fuel") $field_unit = "gas";
            $unit = VehicleEmission::$units[$field_unit] ?? "";

            // bei elektro 0g co2
            if($fuel == "ELECTRICITY" && $field_unit == "co2") $value = 0;

            $emission[] = [
                'label' => $field['label'],
                'filterkey' => $field_slug,
                'meta_key' => $meta_key,
                'meta_value' => $meta_value,
                'value' => $value,
                'unit' => $unit
            ];
        }

        return $emission;
    }

    /**
     * @param $initial_value
     * @return array|string|string[]|null
     */
    private function format_value($initial_value) {
        if(empty($initial_value)) return null;
        $val = str_replace(",",".",$initial_value);
        $val = floatval($val);
        $val = round($val,2);
        $val = number_format($val, 1);
        $val = str_replace(".", ",", $val);

        return $val;
    }

}