<?php

namespace DXIM;

use DXIM\theme\LocationData;
use WPCS_B2B\Company;

class VehiclePDF extends \FPDF {

    protected $show_border = 0;
    protected $filename = null;
    protected $options = null;
    protected $vehicle = null;
    protected $border = 0;

    protected $colorDefault = [50,50,50];
    protected $colorKey = [50,50,50];
    protected $colorContrast = [255,255,255];

    protected $header_image = null;

    protected $color_rect = null;
    protected $color_rect_font = null;
    protected $color_headlines = null;
    protected $color_text = null;
    protected $color_footer = null;
    protected $color_footer_text = null;

    /**
     * VehiclePDF constructor.
     * @param string $orientation
     * @param string $unit
     * @param string $size
     */
    public function __construct($orientation = 'P', $unit = 'mm', $size = 'A4') {
        parent::__construct($orientation, $unit, $size);

        $this->options = Admin::getInstance()->getOptions();
        $this->AliasNbPages();

        $mods = get_theme_mods();
        $color = $mods['wpcs']['pdf_rect_color'] ?? '#333333';
        $this->color_rect = sscanf($color, "#%02x%02x%02x");

        $color = $mods['wpcs']['pdf_rect_font_color'] ?? '#ffffff';
        $this->color_rect_font = sscanf($color, "#%02x%02x%02x");

        $color = $mods['wpcs']['pdf_headline_color'] ?? '#000000';
        $this->color_headlines = sscanf($color, "#%02x%02x%02x");

        $color = $mods['wpcs']['pdf_text_color'] ?? '#000000';
        $this->color_text = sscanf($color, "#%02x%02x%02x");

        $color = $mods['wpcs']['pdf_footer_bg_color'] ?? '#000000';
        $this->color_footer = sscanf($color, "#%02x%02x%02x");

        $color = $mods['wpcs']['pdf_footer_text_color'] ?? '#ffffff';
        $this->color_footer_text = sscanf($color, "#%02x%02x%02x");

        define('EURO', chr(128));
    }

    /**
     * @param \WP_Post $vehicle
     */
    public function create(\WP_Post $vehicle) {

        $this->vehicle = $vehicle;
        $this->prepare_header_image();

        $this->SetMargins(5,5,5);
        $this->SetAutoPageBreak(1,35);
        $this->AddPage();
        $this->setFontColor();
        $this->SetDrawColor(150,150,150);

        // Details
        $this->drawDetails();
        $this->drawSectionLine();

        // Features
        $this->drawFeatures();
        $this->drawSectionLine();

        // Beschreibung
        $this->drawDescription();
        $this->drawSectionLine();

        // Bilder
        if($this->options['vehicle_expose_images'] == "all") $this->drawImages();

        // Pflichttext
        // Umwelt
        $this->AddPage();
        $this->drawEmission();
        $this->drawEmissionText();
        #$this->drawSectionLine();

    }

    private function drawEmissionText() {
        $options = Admin::getInstance()->getOptions();
        if(!isset($options['general_energy_label_text_enable'])) return null;

        $this->SetFont('Arial','',9);
        $this->SetTextColor(150,150,150);

        #$this->Ln();
        $this->Ln(10);
        $this->MultiCell(180,5,$this->_text($options['general_energy_label_text']),$this->border);

    }

    /**
     *
     */
    private function drawEmission() {
        #return;
        $text = null;
        $emission = VehicleData::getVehicleEmission($this->vehicle);
        $this->drawSectionHeadline("Verbrauch & Umwelt");
        $fuel = $this->vehicle->dxim_meta['specific_fuel'][0] ?? "";
        $y = $this->GetY();

        // WLTP
        $text .= "\n".VehicleHtml::drawVehicleEmissionData($this->vehicle, $emission);

        $this->MultiCell(200,5,$this->_text($text),$this->border);

        // Hybrid
        if(strpos($fuel,"HYBRID") !== false) {
            $fields = [
                ['value' => $this->vehicle->dxim_meta['specific_wltp_co2_class_weighted_combined'][0] ?? "", 'label' => 'CO2 - Klasse gewichtet kombiniert'],
                ['value' => $this->vehicle->dxim_meta['specific_wltp_co2_class_low_battery'][0] ?? "", 'label' => 'CO2 - Klasse bei entladener Batterie']
            ];
        }
        // alle anderen
        else $fields = [['value' => $this->vehicle->dxim_meta['specific_wltp_co2_class'][0] ?? "", 'label' => 'CO2 - Klasse kombiniert']];

        $y = $this->GetY() + 10;
        $x = 5;
        foreach($fields as $field) {
            if(!empty($field['value'])) {
                $this->SetY($y);
                $this->SetX($x);
                $this->MultiCell(65,5,$this->_text($field['label']),$this->border);
            }
            $x += 100;
        }

        $x = 5;
        $y = $this->GetY() + 1;
        foreach($fields as $field) {
            if(!empty($field['value'])) {
                $url = Html::getEnergieLabelUrl($field['value']);
                $image = @getimagesize($url);
                if(intval($image[0]) > 0) {
                    $this->SetY($y);
                    $this->SetX($x);
                    $this->CellImage($url, 65, 0, 1);
                }
            }
            $x += 100;
        }

    }

    /**
     * @param $remoteFile
     * @return false|string
     */
    private function grab_file($remoteFile) {
        // can load from URL
        if(ini_get('allow_url_fopen')) return $remoteFile;

        $info = pathinfo($remoteFile);
        $extension = $info['extension'];

        // download image
        require_once(ABSPATH . "wp-admin" . '/includes/file.php');
        $tmpFile = download_url($remoteFile);
        if(!is_wp_error($tmpFile) && is_file($tmpFile)) {
            $localFile = str_replace(".tmp",".".$extension,$tmpFile);
            rename($tmpFile,$localFile);
            return $localFile;
        }
        else return false;
    }

    /**
     *
     */
    private function drawDetails() {
        $this->drawSectionHeadline('Details');
        $images = VehicleData::getVehicleImages($this->vehicle);
        $url = $images['large'][0];

        // site_url() heißt bild liegt lokal
        if(strpos($url, site_url()) !== false) {
            $dirInfo = wp_get_upload_dir();
            $localFile = str_replace($dirInfo['baseurl'],$dirInfo['basedir'],$url);
            if(!is_file($localFile)) $localFile = null;
        }
        // import from url
        else {
            $localFile = $this->grab_file($url);
        }

        $y = $this->GetY();

        // Image
        $this->CellImage($localFile,90,0,1);
        $y_after_image = $this->GetY();

        // Price
        $this->Ln(2);
        $this->drawPrice();

        // specifics
        $this->SetY($y);
        $this->drawSpecifics();
        $y_after_spacifics = $this->GetY();

        if(($y_after_image+10) > $y_after_spacifics) $this->SetY($y_after_image+10);
    }

    /**
     *
     */
    private function drawSpecifics() {
        $x_column2 = 100;
        $specifics = VehicleData::getVehicleFields($this->vehicle,'single');
        #wpcs_p($specifics);

        $z = 0;
        foreach($specifics as $tmp) {
            if(empty($tmp['display_value'])) continue;
            $this->SetX($x_column2);

            $this->Line($this->GetX(), $this->GetY(),$this->GetX()+100, $this->GetY());

            $this->SetFont('Arial','B',10);
            $this->Cell(45,7,$this->_text($tmp['label']),$this->border);

            $this->SetFont('Arial','',10);
            $this->MultiCell(55,7,$this->_text($tmp['display_value']),$this->border, 1);
            $z++;
        }
    }

    private function drawPrice() {
        $price = VehicleData::getVehiclePrice($this->vehicle);
        $price['display_price_brutto'] = str_replace("&euro;",EURO,$price['display_price_brutto']);
        $price['display_price_netto'] = str_replace("&euro;",EURO,$price['display_price_netto']);

        $x_column2 = 105;
        #wpcs_p($price);
        $this->SetFont('Arial','',12);
        $this->Cell(40,10,$this->_text("Preis"),$this->border,0);
        $this->SetFont('Arial','',15);
        $this->setTextColor($this->color_rect_font[0], $this->color_rect_font[1], $this->color_rect_font[2]);
        $this->SetFillColor($this->color_rect[0], $this->color_rect[1], $this->color_rect[2]);
        $this->SetX(35);


        // Preis aus Anfrage
        if($price['price_type'] == 'ON_REQUEST') {
            $this->Cell(60,8,"Preis auf Anfrage",$this->border,1,'R', 1);
        }

        // keine MwSt.
        elseif(!$price['vatable']) {
            $this->Cell(60,8,$price['display_price_brutto'],$this->border,1,'R', 1);
            $this->SetFont('Arial','',10);
            $this->SetX(35);
        }

        // inkl. MwSt.
        else {

            // netto
            if($price['price_display'] == 'netto') {
                $this->Cell(60,8,$price['display_price_netto'],$this->border,1,'R', 1);
            }

            // brutto
            elseif($price['price_display'] == "brutto") {
                $this->Cell(60,8,$price['display_price_brutto'],$this->border,1,'R',1);
                $this->SetFont('Arial','',10);
                $this->SetX(35);
                $this->Cell(60,8,$price['vat_msg'],$this->border,1,'R',1);
            }

            elseif($price['price_display'] == "both") {
                $this->Cell(60,8,$price['display_price_brutto'],$this->border,1,'R',1);
                $this->SetFont('Arial','',10);
                $this->SetX(35);
                $this->Cell(60,8,$price['display_price_netto'].' (Netto), '.$price['vat_msg'],$this->border,1,'R',1);
            }
        }
        $this->setFontColor();
    }

    private function drawDescription() {
        $this->drawSectionHeadline('Fahrzeugbeschreibung');

        $content = $this->vehicle->post_content;
        $content = strip_tags($content);
        $content = str_replace("\\",'<br />',$content);
        $content = preg_replace('/(\*\*)(.*?)(\*\*)/', "<strong>$2</strong>", $content);
        $content = str_replace("*",'-',$content);
        $content = str_replace("---",'----',$content);
        $content = str_replace("----",'<hr />',$content);
        $content = str_replace(["‘","„", "“","–"],['"','"','"',"-"],$content);
        $content = htmlspecialchars_decode($content,ENT_QUOTES);


        $a=preg_split('/<(.*)>/U',$content,-1,PREG_SPLIT_DELIM_CAPTURE);
        #wpcs_p($a);
        $previousTag = null;
        foreach($a as $text) {

            if($text == "strong") {
                $this->SetFont('Arial','B',10);
            }
            elseif($text == "/strong") {
                $this->SetFont('Arial','',8);
            }
            elseif($text == "br /") {
                if($previousTag != $text) $this->Write(5,"\n");
            }
            elseif($text == "hr /") {
                $this->Write(5,"\n");
                $this->Write(5,"____________________________________________________________");
                $this->Write(5,"\n");
            }
            elseif(empty(trim($text)) || trim($text) == "-") { continue;  }

            // TEXT
            else {
                $text = str_replace(["- -", "--"], ["--", "-"], $text);
                $this->Write(5,$this->_text($text));
            }

            $previousTag = $text;

        }


    }


    private function drawFeatures() {
        $this->drawSectionHeadline('Ausstattung');

        $features = VehicleData::getVehicleFeatures($this->vehicle);

        $cells = 3;
        $z = 1;
        $cw = (($this->GetPageWidth() - 10) / $cells);
        $this->SetDrawColor(200,200,200);

        $layout = Admin::getInstance()->getOptions('single_vehicle_features_layout') ?? null;
        if($layout == "grouped") {
            $categories = Features::$featureCategories;
            foreach($categories as $cat_key => $cat_name) {
                if(!empty($group) && $cat_key !== $group) continue;
                if(!isset($features[$cat_key])) continue;

                $this->SetFont('Arial','',12);
                $this->Write(8,$this->_text($cat_name));
                $this->Ln(8);

                $this->SetFont('Arial','',8);
                foreach($features[$cat_key] as $feature_key => $feature_values) {
                    if(count($feature_values) > 1) {
                        $tmpFeature = implode(", ",$feature_values);
                    }
                    else {
                        $tmpFeature = $feature_values[0];
                    }
                    $ln = ($z % $cells == 0) ? 1 : 0;
                    $this->Cell($cw,6,$this->_text("- ".$tmpFeature),0,$ln);
                    $z++;
                }
                $this->Ln(10);
            }

        }
        else {
            $this->SetFont('Arial','',8);

            foreach($features as $feature) {
                if(empty($feature)) continue;
                if(is_array($feature)) {
                    foreach($feature as $tmpFeature) {
                        $ln = ($z % $cells == 0) ? 1 : 0;
                        $this->Cell($cw,6,$this->_text("- ".$tmpFeature),0,$ln);
                        $z++;
                    }
                }
                else {
                    $ln = ($z % $cells == 0) ? 1 : 0;
                    $this->Cell($cw,6,$this->_text("- ".$feature),0,$ln);
                    $z++;
                }
            }
        }

    }

    /**
     * @return |null
     */
    private function drawImages() {
        $images = VehicleData::getVehicleImages($this->vehicle);
        if(empty($images)) return null;

        $this->drawSectionHeadline('Fahrzeugbilder');

        $cells = 3;
        $gutter = 3;
        $space = ($cells * $gutter) / ($cells - 1);
        $cw = (($this->GetPageWidth() - 10) / $cells) - $gutter;
        $ch = $cw - 10;
        $z = 1;

        foreach($images['small'] as $url) {

            $ln = ($z % $cells == 0) ? 1 : 0;

            if(($this->GetY() + $ch) > ($this->GetPageHeight() - 40)) $this->AddPage();

            $this->CellImage($url,$cw,$ch,$ln);
            if(!$ln) $this->Cell($space, $ch,"",$this->border,0);
            else $this->Ln($space);

            $z++;
        }
    }

    /**
     * @param $url
     * @param $cw
     * @param int $ch
     * @param $ln
     * @ToDo Bildquelle abfangen falls ncht vorhanden
     */
    private function CellImage($url,$cw,$ch = 0,$ln = 0) {
        if(empty($url)) return false;

        if($ch == 0) {
            $image_info = getimagesize($url);
            $f = $image_info[0] / $image_info[1];
            $ch = $cw / $f;
        }

        try {
            $this->Image($url,$this->GetX(),$this->GetY(),$cw);
            $this->Cell($cw, $ch,null,$this->border,$ln);
        } catch(\Exception $e) {
        }

    }

    /**
     * @return false
     */
    private function prepare_header_image() {
        $location_id = $this->vehicle->dxim_meta['ad_location'][0] ?? 0;
        $logo_id = get_term_meta($location_id, 'logo', true);
        if(empty($logo_id)) {
            $term = get_term($location_id);
            $logo_id = get_term_meta($term->parent ?? 0, 'logo', true);
        }
        $url = wp_get_attachment_image_url($logo_id, 'medium');

        // fallback auf altes logo aus den settings
        if(empty($url)) {
            $url = apply_filters(DXIM_FILTER_DEALER_LOGO_URL, $this->options['autohaus_logo']);
        }

        if(empty($url)) return false;

        if(preg_match('/(http|https):/',$url)) $url = $this->grab_file($url);
        $this->header_image = $url;
    }

    /**
     *
     */
    public function Header() {

        if(!empty($this->header_image)) {

            $image_info = @getimagesize($this->header_image);
            if($image_info[0] > 1) {
                list($w, $h) = $image_info;
                $image_w = $image_h = null;
                $image_max_h = 20;
                $adjustHeight = false;

                if($w > $h) {
                    $image_w = 50;
                    $f = $image_info[0] / $image_w;
                    $image_h = $image_info[1] / $f;
                    if($image_h > $image_max_h) $adjustHeight = true;
                }
                else {
                    $adjustHeight = true;
                }

                if($adjustHeight) {
                    $image_h = 20;
                    $f = $image_info[1] / $image_h;
                    $image_w = $image_info[0] / $f;
                }


                $this->Image($this->header_image,200-$image_w,8,$image_w, $image_h);
            }
        }

        // Headline
        $this->SetFont('Arial','',12);
        $this->setFontColor();

        #$this->Cell($image_w+10);
        $this->Cell(0,6,$this->_text('Fahrzeug - Exposé'),$this->border,1);

        // Fahrzeugnummer
        $this->setFontColor();
        $this->SetFont('Arial','',10);
        #$this->Cell($image_w+10);
        $this->Cell(0,6,$this->_text('Fahrzeug Nr.: '.$this->vehicle->dxim_meta['seller_inventory_key'][0]),$this->border,1);

        // Fahrzeugname
        $this->Ln(3);
        $yStart = $this->GetY();
        $this->SetFont('Arial','B',12);
        #$this->Cell($image_w+10);
        $this->setTextColor($this->color_rect_font[0], $this->color_rect_font[1], $this->color_rect_font[2]);
        $this->SetFillColor($this->color_rect[0], $this->color_rect[1], $this->color_rect[2]);
        $this->MultiCell(130,10,$this->_text($this->vehicle->post_title),$this->border,null,1);
        $this->Rect(0,$yStart,5,round($this->GetY()-$yStart),'F');

        // Ende Header
        $this->Line(0,$this->GetY()+5,$this->GetPageWidth(),$this->GetY()+5);
        $this->Ln(15);

    }


    /**
     *
     */
    public function Footer() {
        $ld = new LocationData();
        $location = $ld->getById($this->vehicle->dxim_meta['ad_location'][0]);

        $this->SetY(-30);

        $this->SetFillColor($this->color_footer[0],$this->color_footer[1], $this->color_footer[2]);
        $this->Rect(0,$this->GetY()-5,210,40,'F');

        // Headline
        $this->SetFont('Arial','B',16);
        $this->setTextColor($this->color_footer_text[0],$this->color_footer_text[1],$this->color_footer_text[2]);
        $this->Cell(0,10,$this->_text("Kontakt"),$this->border,1);


        // line between address & contact
        $this->Line(80,$this->GetY(),80,$this->GetPageHeight());

        // Arial italic 8
        $this->SetFont('Arial','',10);

        // Address
        if(!empty($location)) {
            $meta = get_term_meta($location->term_id);

            $street = $meta['street'][0] ?? "";
            $zip = $meta['zipcode'][0] ?? "";
            $city = $meta['city'][0] ?? "";
            $address = apply_filters(DXIM_FILTER_PDF_FOOTER_COL1_TEXT, $location->name."\n".$street."\n".$zip." ".$city);

            $this->MultiCell(80,5,$this->_text($address),$this->border);

            // Kontakt
            $this->SetY(-20);
            $this->Cell(80);

            $tel = $meta['phone_base'][0] ?? "";
            $tel .= $meta['phone'][0] ?? "";
            $email = $meta['email'][0] ?? "";

            $contact = apply_filters(DXIM_FILTER_PDF_FOOTER_COL2_TEXT,"Tel.: ".$tel."\n".'E-Mail: '.$email."\n");
            $this->MultiCell(80,5,$this->_text($contact),$this->border);
        }

        $this->SetY(-10);
        // spacer
        $this->Cell(140);

        // Page number
        $this->SetFont('Arial','I',10);
        $this->Cell(60,10,'Seite '.$this->PageNo().'/{nb}',$this->border,0,'R');

    }


    /**
     *
     */
    private function drawSectionLine() {
        $this->Ln(10);
        $this->Line(0,$this->GetY(),$this->GetPageWidth(),$this->GetY());
        $this->Ln(5);
    }

    /**
     * @param $text
     */
    private function drawSectionHeadline($text) {
        $this->SetFont('Arial','B',15);
        $this->setFontColor('headline');
        $this->Cell(0,10,$this->_text($text),$this->border,1);
        $this->Ln(5);

        $this->setFontColor();
        $this->SetFont('Arial','',10);
    }

    /**
     * @param string $type
     */
    private function setFontColor($type = 'normal') {
        if($type == 'normal') {
            $color = $this->color_text;
        }
        elseif($type == 'headline') {
            $color = $this->color_headlines;
        }
        elseif($type == 'key') {
            $color = $this->colorKey;
        }
        elseif($type == 'contrast') {
            $color = $this->colorContrast;
        }

        list($r,$g,$b) = $color;
        $this->SetTextColor($r,$g,$b);
    }

    /**
     * @param $text
     * @return string
     */
    private function _text($text) {
        $text = str_replace('€','#EURO#',$text);
        $text = strip_tags($text);
        $text = html_entity_decode($text);
        $text = mb_convert_encoding($text, 'ISO-8859-1', 'UTF-8');
        #$text = str_replace("#EURO#",EURO,$text);
        return $text;
    }

}

