<?php

namespace DXIM;

use function WPMailSMTP\Vendor\GuzzleHttp\Promise\queue;

class CSVAdapter {

    protected static $instance = null;

    private $csv_file = null;
    private $config_file = null;
    private $zip_name = null;

    private $ident_key_index = null;
    private $error = null;

    private $config_data = null;
    private $csv_data = null;

    /**
     *
     * @return CSVAdapter
     */
    public static function getInstance() {
        if(self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     *
     */
    public function __construct() {
        CSVExtendedImportHooks::getInstance();

        add_action(DXIM_CORE_AFTER_IMPORT,[&$this,'_cleanup'],10);

        add_filter('dxim_core_csv_importer_get_image_links',[&$this,'_get_image_links'],100,2);
    }

    /**
     * @return null
     */
    public function getError() { return $this->error; }



    /**
     * @return array|null[]|null
     */
    public function getConfigData() {
        return $this->config_data;
    }


    /**
     * @param $account
     * @return array|false|null
     */
    public function initAccount($account, $ignore_csv = false) {
        $this->csv_file = stripslashes($account['path_csv']);
        $this->config_file = stripslashes($account['path_config']);
        $this->zip_name = stripslashes($account['zip_name']);

        // falls zip file vorhanden, csv datei entpacken
        if(!$ignore_csv) $this->_extract_csv();

        if(!is_file($this->csv_file)) {
            $this->error = "CSV Datei ".$this->csv_file." nicht gefunden";
            return false;
        }
        if(!is_file($this->config_file)) {
            $this->error = "Config Datei ".$this->config_file." nicht gefunden";
            return false;
        }

        if(!$this->_get_config_data()) return false;

        $this->ident_key_index = $this->config_data['index_key'];
        return true;
    }

    /**
     * @return bool|void
     */
    private function _extract_csv() {
        if(empty($this->zip_name)) return false;

        $path = pathinfo($this->csv_file);
        $zip_name = $this->zip_name;
        $dir = $path['dirname']."/";
        $zipFile = $dir.$zip_name;

        if(!is_file($zipFile)) return false;

        if(!class_exists('ZipArchive')) {
            Sync::log("Zip Datei kann nicht entpackt werden: Klasse ZipArchive nicht gefunden");
            return false;
        }

        $zip = new \ZipArchive();
        if ($zip->open($zipFile) === TRUE) {
            for ($i = 0; $i < $zip->numFiles; $i++) {
                $filename = $zip->getNameIndex($i);

                if(strpos($filename, '.csv') !== false) {
                    $zip->extractTo($dir, $filename);
                    Sync::log("CSV Datei entpackt");
                }
            }
            $zip->close();
            return true;
        }
    }


    /**
     * Wenn zip file in angegebenem Ordner existiert (egal welcher Name)
     * wird dieses entpackt und gelöscht
     * @param $csvFile
     * @return false|void
     */
    private function _extract_images($image_id) {

        $path = pathinfo($this->csv_file);
        $zip_name = $this->zip_name;
        $dir = $path['dirname']."/";
        $images_dir = FilesystemImport::getInstance()->get_import_dir();
        $zipFile = $dir.$zip_name;
        $files_extracted = [];

        if(!is_file($zipFile)) return false;
        if(!class_exists('ZipArchive')) return false;

        $pattern = '/^'.$image_id.'_.+\.jpg$/';
        $zip = new \ZipArchive();

        if ($zip->open($zipFile) === TRUE) {
            for ($i = 0; $i < $zip->numFiles; $i++) {
                $filename = $zip->getNameIndex($i);

                if(preg_match($pattern, $filename)) {
                    $zip->extractTo($images_dir, $filename);
                    $files_extracted[] = $filename;
                }
            }

            $zip->close();

//                // delete zip file
//                if(unlink($zipFile)) {
//                    Sync::log("Zip-Datei gelöscht");
//                }

        } else {
            Sync::log("Fehler beim öffnen der Zip Datei");
            return false;
        }

        return $files_extracted;

    }

    /**
     * @param $pm
     * @return string[]
     * @return string[]
     */
    public function _get_image_links($pm, $config) {
        if(!isset($pm['image_links'])) return $pm;
        return explode(",", $pm['image_links']);
    }

    /**
     * @param $index
     * @return array|false
     */
    private function _get_csv_line($index) {
        if(!is_file($this->csv_file)) {
            $this->error = "Konnte CSV Datei nicht finden";
            return false;
        }

        $from_encoding = $this->config_data['encoding'] ?? "Windows-1252";

        $file = fopen($this->csv_file, 'r');
        for ($i = 0; $i <= $index; $i++) {
            $lineTmp = fgets($file);
            $line = mb_convert_encoding($lineTmp, 'UTF-8', $from_encoding);
        }
        fclose($file);

        if(empty($line)) return null;

        $data = str_getcsv($line, ";"); //parse the items in rows
        return $data;
    }


    /**
     * @return mixed
     */
    private function _get_config_data() {
        $this->config_data = null;

        if(!is_file($this->config_file)) {
            $this->error = "Config File nicht gefunden";
            return false;
        }

        $json = file_get_contents($this->config_file);
        $data = json_decode($json,true);
        $columns = [];

        foreach($data['columns'] as $index => $col) {
            if(isset($col['callback'])) {
                foreach($col['callback'] as $col2) {
                    $columns[$col2['name']][$index] = $col2;
                    unset($columns[$col2['name']][$index]['name']);
                }
            }
            else {
                $columns[$col['name']][$index] = $col;
                unset($columns[$col['name']][$index]['name']);
            }
        }

        foreach($columns as $field_name => $array) {
            if(count($array) < 1) continue;
            uasort($array,function($a, $b) {
                if($a['priority'] == $b['priority']) return 0;
                return ($a['priority'] > $b['priority']) ? -1 : 1;
            });

            $columns[$field_name] = $array;
        }
        $data['columns'] = $columns;

        $this->config_data = $data;
        return true;
    }

    /**
     * @param $importer_id
     * @param $account
     * @return bool|null[]
     */
    public function setup($importer_id, $account) {
        if(!$this->initAccount($account)) {
            Sync::log($this->error);
            return false;
        }

        $count = $this->_count();
        Import::getInstance()->vehicles_count($count);

        return true;
    }

    private function _count() {
        // Öffnen der CSV-Datei
        $time_start = microtime(true);

        $file = fopen($this->csv_file, 'r');
        $numRows = 0;
        while (!feof($file)) {
            $row = fgetcsv($file);
            if (!empty($row)) {
                $numRows++;
            }
        }
        fclose($file);

        Sync::extLog("Dauer CSV zählen: ".(microtime(true) - $time_start));

        return $numRows;
    }


    /**
     * @param $key
     * @param $account
     * @return false|void
     */
    public function getVehicle($index, $account, $preview = false) {

        if(!$this->initAccount($account, true)) return 0;
        $config = &$this->config_data;

        if(isset($config['ignore_headline']) && $index == 0) {
            Sync::log("ignoriere index ".$index);
            return 0;
        }

        $data = $this->_get_csv_line($index);

        // keine daten in position in csv
        if(is_null($data)) return 1;


        $ad_key = hash('md5',$data[$this->ident_key_index].'-'.$account['user']);
        $hash = hash("md5",implode(";",$data));

        /**
         * prüfe was mit Fahrzeug zu tun ist
         */
        $v = new Vehicle();
        $vehicle = $v->createImportDataset($ad_key, $hash, $data[$this->ident_key_index]);

        if(is_null($vehicle['import_data']['do']) && !$preview) return $vehicle;

        /**
         * post_meta aufbauen (custom fields für wp_postmeta tabelle)
         */
        $filter = VehicleFilter::getFilters();
        $features = Features::getFeatures();
        $pm = [
            'ad_key' => $ad_key,
            'import_hash' => $hash,
            'ad_account' => $account['user']
        ];
        foreach($config['columns'] as $field_name => $cols) {

            // im filter array
            if(isset($filter[$field_name])) $meta_key = $filter[$field_name]['meta_query']['key'] ?? $field_name;

            // im feature array
            elseif(isset($features[$field_name])) {
                $meta_key = "feature_".strtolower($field_name);
            }

            // field_name nicht gefunden
            else { $meta_key = $field_name; }

            if(empty($meta_key)) continue;

            $meta_value = null;
            foreach($cols as $colIndex => $colInfo) {
                // kein Wert in csv vorhanden
                if(!isset($data[$colIndex])) continue;

                $value = $data[$colIndex];

                // import value by map to internal value
                if(isset($colInfo['mapping'])) {
                    if(!isset($colInfo['mapping'][$value])) continue;
                    $meta_value = $colInfo['mapping'][$value];
                }

                // import value itself
                else {
                    $meta_value = $value;
                }

                // wurde kein Wert gefunden => überspringen
                if($meta_value === "") continue;

                // sind mehrere cols zum angegebenen key vorhanden
                // a) gibt es eine priorität wird nur der key mit der höchsten genommen
                // b) gibt es keine werden alle werte genommen
                if(count($cols) > 1) {

                    if(!empty($colInfo['priority'])) {
                        $pm[$meta_key] = $meta_value;
                        break;
                    }
                    else $pm[$meta_key][] = $meta_value;

                }
                else $pm[$meta_key] = $meta_value;

            }

        }

        /**
         * zugehörige WPCS location mappen
         */
        $location_key = $data[$config['location']['index']];
        $location_mapping = get_option('wpcs_csv_locations_'.$account['user']);
        $pm['ad_location'] = $location_mapping[$location_key] ?? $location_key;

        /**
         * manuelle Felder ergänzen
         */
        $cfields = $this->getCustomFields($account['user']);
        foreach($cfields as $index => $field) {
            if(!isset($data[$index])) continue;
            $pm[$field['name']] = $data[$index];
        }

        /**
         * copy vehicle images
         */
        $all_images = [];
        // Wenn eine Bild ID angegeben wurde,
        // Bilder anhand der ID im Zip File suchen und extrahieren
        // anschließend Bilder umbenennen
        if(!empty($pm['image_id'])) {

            // 1) Extrahiere Bilder aus Zip File
            $files = $this->_extract_images($pm['image_id']);
            if(!is_array($files)) $files = [];
            natsort($files);

            // 2) Bename die Bilder um zur passenden image_id
            $images_dir = trailingslashit(FilesystemImport::getInstance()->get_import_dir());
            $z = 1;
            foreach($files as $file) {
                $info = pathinfo($file);
                $new_name = $ad_key."_".$z.".".$info['extension'];
                rename($images_dir.$file, $images_dir.$new_name);
                $all_images[] = $new_name;
                $z++;
            }
        }
        // Wurde keine Bild - ID angegeben, Hook feuern
        // so können z.b. Bild-Links ausgewertet werden
        else {
            $pm['no_images'] = true; // falls der hook angewendet wird, sollte dieser Werte nicht mehr vorhanden sein => nur dann bilder importieren
            $image_links = apply_filters('dxim_core_csv_importer_get_image_links', $pm, $config);
            if(!empty($image_links) && is_array($image_links) && !isset($image_links['no_images'])) {
                $z = 1;
                $images_dir = trailingslashit(FilesystemImport::getInstance()->get_import_dir());
                foreach($image_links as $url) {
                    if(empty($url)) continue;
                    $parsed_url = parse_url($url);
                    $url = $parsed_url['scheme'] . "://" . $parsed_url['host'] . $parsed_url['path'];
                    $info = pathinfo($url);
                    $name = $ad_key."_".$z.".".$info['extension'];
                    $all_images[] = FilesystemImport::getInstance()->importFromUrl($url, $images_dir, $name);
                    $z++;
                }
            }

        }

        // 3) suche alle final zur image_id passenden Bilder im DS um Bildliste dem Fahrzeug als Meta zuzuweisen
        #$all_images = FilesystemImport::getInstance()->getImagesByName($ad_key);
        $images = [];
        foreach($all_images as $tmp) {
            $img = pathinfo($tmp);
            if($img['extension'] == "jpg" || $img['extension'] == "jpeg") $images[] = $img['basename'];
        }

        #wpcs_p($images);
        $pm['images']['big'] = $images;

        // wir merken uns  den originalen image_id
        $pm['image_id_external'] = $pm['image_id'] ?? "";
        // nutzen aber wpcs intern den ad_key als image_id => wegen händler übergreifender eindeutigkeit
        $pm['image_id'] = $ad_key;

        /**
         * dynamic hook foreach config file
         */
        if(isset($config['filter_hook'])) $pm = apply_filters('dxim_core_csv_importer_'.$config['filter_hook'],$pm, $data, $config);
        $vehicle['post_meta'] = $pm;

        /**
         * post_data aufbauen (Bestandteile von WP_POST in wp_posts tabelle)
        */
        $pd = [];

        // post title
        $description = sanitize_text_field($pm['model_description']);
        $pd['post_title'] = Make::_get($pm['make'])." ".$description;

        // post description
        $pd['post_content'] = sanitize_text_field($data[$config['description']['index']]);

        // post slug including key
        $pd['post_name'] = Html::sanitize_title($pd['post_title'].'-'.$ad_key);

        $vehicle['post_data'] = $pd;

        return $vehicle;
    }




    /**
     * @param $index
     * @param $key
     * @return false|void
     */
    public function addNewCustomField($index, $key, $account) {
        if(empty($index) || empty($key) || empty($account)) return false;

        $fields = $this->getCustomFields($account);
        $fields[$index] = ['name' => sanitize_title($key)];

        return $this->saveCustomFields($fields, $account);
    }


    private function saveCustomFields($fields, $account) {
        return update_option("wpcs_csv_cf_".$account,$fields);
    }


    public function deleteCustomField($index, $account) {
        if(empty($index)) return;

        $fields = $this->getCustomFields($account);
        unset($fields[$index]);

        return $this->saveCustomFields($fields, $account);
    }

    /**
     * @return false|mixed|void
     */
    public function getCustomFields($account) {
        $fields = get_option("wpcs_csv_cf_".$account);
        if(!is_array($fields)) return [];

        ksort($fields, SORT_NUMERIC);
        return $fields;
    }


    public function _cleanup() {
        $extensions = ['png', 'jpg', 'jpeg'];
        $options = Admin::getInstance()->getOptions();
        $folders = [];
        if(!isset($options['accounts'])) return;

        foreach($options['accounts'] as $account) {
            if($account['type'] == "csv") $folders[] = stripslashes(pathinfo($account['path_csv'])['dirname']);
        }

        if(!empty($options['csv_import_options']['import_folder'])) {
            $folders[] = stripslashes($options['csv_import_options']['import_folder']);
        }

        foreach($folders as $folder) {
            if(!is_dir($folder)) continue;
            $d = dir($folder);
            while (false !== ($entry = $d->read())) {
                $file = pathinfo($entry);
                if(!in_array(strtolower($file['extension']), $extensions)) continue;
                unlink(trailingslashit($folder).$entry);
            }
            $d->close();
        }
    }



}